package com.cash_gl.cash_gl_plugin;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.util.Base64;
import android.util.Log;

import androidx.annotation.NonNull;

import com.mpurse.pay.View.Entity.PayEntity;
import com.mpurse.pay.View.util.MYPauseConstants;
import com.mpurse.pay.mPausePayPresenter;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import ai.advance.liveness.lib.LivenessResult;
import ai.advance.liveness.sdk.activity.LivenessActivity;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;

/**
 * CashGlPlugin
 */
public class CashGlPlugin implements PluginRegistry.ActivityResultListener, MethodCallHandler {

    private static final String CHANNEL = "com.cash_gl.cash_gl_plugin";

    private static final int LIVENESS_REQUEST_CODE = 10110;

    private Activity activity;

    // callback
    private PayWithMpurseCallback payWithMpurseCallback;

    private LivenessCallback livenessCallback;

    private CashGlPlugin(Activity activity) {
        this.activity = activity;
    }

    /**
     * Plugin registration.
     */
    public static void registerWith(PluginRegistry.Registrar registrar) {
        System.out.println("====registerWith");
        Log.i("CashGlPlugin", "====registerWith");
        final CashGlPlugin plugin = new CashGlPlugin(registrar.activity());
        final MethodChannel channel = new MethodChannel(registrar.messenger(), CHANNEL);
        channel.setMethodCallHandler(plugin);
        registrar.addActivityResultListener(plugin);
    }


    @Override
    public void onMethodCall(@NonNull final MethodCall call, @NonNull final Result result) {
        if ("payWithMpurse".equals(call.method)) {
            // 初始化 callback
            payWithMpurseCallback = new PayWithMpurseCallback(result);
            Log.i("CashGlPlugin", this + "");
            Log.i("CashGlPlugin", "onMethodCall.payWithMpurse");
            Log.i("CashGlPlugin", activity + "");
            payWithMpurse((String) call.argument("amount"), (String) call.argument("txnId"),
                    (String) call.argument("mpQueryId"), (String) call.argument("partnerId"),
                    (String) call.argument("createTime"), (String) call.argument("payerVa"),
                    (int) call.argument("code"));
        } else if ("liveness".equals(call.method)) {
            livenessCallback = new LivenessCallback(result);
            liveness();
        } else {
            result.notImplemented();
        }
    }

    @Override
    public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
        Log.i("CashGlPlugin", "onActivityResult: requestCode=" + requestCode + ", resultCode=" + resultCode);
        switch (requestCode) {
            case MYPauseConstants.RESULT_TYPE_REQUEST_CODE: {
                if (payWithMpurseCallback != null && resultCode == MYPauseConstants.TYPE) {
                    int back = data.getIntExtra("BACK", 0);
                    if (back == MYPauseConstants.RESULT_BACK_CODE) {
                        Log.i("CashGlPlugin", "RESULT_BACK_CODE");
                    }
                    Log.i("CashGlPlugin", this + "");
                    payWithMpurseCallback.sucess(back);
                }
                break;
            }
            case LIVENESS_REQUEST_CODE: {
                if (livenessCallback != null) {
                    Log.i("CashGlPlugin", "LivenessResult.isSuccess(): " + LivenessResult.isSuccess());
                    if (LivenessResult.isSuccess()) {// 活体检测成功
                        String livenessId = LivenessResult.getLivenessId();// 本次活体id
                        Bitmap livenessBitmap = LivenessResult.getLivenessBitmap();// 本次活体图片
                        Log.i("CashGlPlugin", "livenessId: " + livenessId);
                        livenessCallback.sucess(livenessId, livenessBitmap);
                    } else {// 活体检测失败
                        String errorCode = LivenessResult.getErrorCode();// 失败错误码
                        String errorMsg = LivenessResult.getErrorMsg();// 失败原因
                        Log.i("CashGlPlugin", "errorCode=" + errorCode + ", errorMsg=" + errorMsg);
                        livenessCallback.failed(errorCode, errorMsg);
                    }
                }
                break;
            }
            default:
        }
        return false;
    }

    private void liveness() {
        Intent intent = new Intent(activity, LivenessActivity.class);
        activity.startActivityForResult(intent, LIVENESS_REQUEST_CODE);
    }

    private void payWithMpurse(String amount, String txnId, String mpQueryId,
                               String partnerId, String createTime, String payerVa,
                               int code) {
        Log.i("CashGlPlugin", "before payWithMpurse");
        Log.i("CashGlPlugin", amount);
        Log.i("CashGlPlugin", txnId);
        Log.i("CashGlPlugin", mpQueryId);
        Log.i("CashGlPlugin", partnerId);
        Log.i("CashGlPlugin", createTime);
        Log.i("CashGlPlugin", payerVa);
        Log.i("CashGlPlugin", code + "");

        mPausePayPresenter mPayPresenter = new mPausePayPresenter();
        // 设置标题背景色
        mPayPresenter.SetTitleColor(Color.parseColor("#FF3333"));
        // 设置按钮颜色及Tab颜色
        mPayPresenter.SetButtonColor(Color.parseColor("#FF3333"));

        PayEntity payEntity = new PayEntity();
        payEntity.setAmount(amount);
        payEntity.setTxnId(txnId);
        payEntity.setMpQueryId(mpQueryId);
        payEntity.setPartnerId(partnerId);
        payEntity.setCreateTime(createTime);
        payEntity.setPayerVA(payerVa);
        mPayPresenter.getNetBankForResult(activity, payEntity, code);
        Log.i("CashGlPlugin", "after payWithMpurse");
    }

    class PayWithMpurseCallback {

        private Result result;

        public PayWithMpurseCallback(Result result) {
            this.result = result;
        }

        public void sucess(int resultCode) {
            Map<String, String> map = new HashMap<>();
            map.put("resultCode", resultCode + "");
            result.success(map);
        }

    }

    class LivenessCallback {
        private Result result;

        public LivenessCallback(Result result) {
            this.result = result;
        }

        public void sucess(String livenessId, Bitmap livenessBitmap) {
            String livenessBitmap64 = null;
            // bitmap to base64
            ByteArrayOutputStream byteArrayOutputStream = null;
            try {
                byteArrayOutputStream = new ByteArrayOutputStream();
                livenessBitmap.compress(Bitmap.CompressFormat.JPEG, 100, byteArrayOutputStream);
                byte[] byteArray = byteArrayOutputStream.toByteArray();
                livenessBitmap64 = Base64.encodeToString(byteArray, Base64.DEFAULT);
            } catch (Exception e) {
                Log.w("CashGlPlugin", e);
            } finally {
                if (byteArrayOutputStream != null) {
                    try {
                        byteArrayOutputStream.close();
                    } catch (IOException e) {
                        Log.w("CashGlPlugin", e);
                    }
                }
            }

            if (livenessBitmap64 != null) {
                Map<String, String> map = new HashMap<>();
                map.put("success", "true");
                map.put("livenessId", livenessId);
                map.put("livenessBitmap64", livenessBitmap64);
                result.success(map);
            }
        }

        public void failed(String errorCode, String errorMsg) {
            Map<String, String> map = new HashMap<>();
            map.put("success", "false");
            map.put("errorCode", errorCode);
            map.put("errorMsg", errorMsg);
            result.success(map);
        }

    }
}
