import 'dart:async';

import 'package:flutter/services.dart';

class CashGlPlugin {
  static const MethodChannel _channel =
      const MethodChannel('com.cash_gl.cash_gl_plugin');

  Future<PayWithMpurseResult> payWithMpurse(String amount, String txnId,
      String mpQueryId, String partnerId, String createTime, String payerVa,
      {code = 1}) async {
    var arguments = Map();
    arguments['amount'] = amount;
    arguments['txnId'] = txnId;
    arguments['mpQueryId'] = mpQueryId;
    arguments['partnerId'] = partnerId;
    arguments['createTime'] = createTime;
    arguments['payerVa'] = payerVa;
    arguments['code'] = code;
    final Map<dynamic, dynamic> map =
        await _channel.invokeMethod('payWithMpurse', arguments);
    return map != null ? new PayWithMpurseResult.fromMap(map) : null;
  }

  Future<LivenessResult> liveness() async {
    final Map<dynamic, dynamic> map = await _channel.invokeMethod('liveness');
    return map != null ? new LivenessResult.fromMap(map) : null;
  }

}

class LivenessResult {
  LivenessResult(
      {this.success,
      this.livenessId,
      this.livenessBitmap64,
      this.errorCode,
      this.errorMsg});

  factory LivenessResult.fromMap(Map<dynamic, dynamic> map) =>
      new LivenessResult(
        success: map['success'],
        livenessId: map['livenessId'],
        livenessBitmap64: map['livenessBitmap64'],
        errorCode: map['errorCode'],
        errorMsg: map['errorMsg'],
      );

  final String success;

  // sucess=true
  final String livenessId;

  // success=true
  final String livenessBitmap64;

  // success=false
  final String errorCode;

  // success=false
  final String errorMsg;

  @override
  String toString() =>
      'LivenessResult: $success,$livenessId,$errorCode,$errorMsg,$livenessBitmap64';
}

class PayWithMpurseResult {
  PayWithMpurseResult({this.resultCode});

  factory PayWithMpurseResult.fromMap(Map<dynamic, dynamic> map) =>
      new PayWithMpurseResult(
        resultCode: map['resultCode'],
      );

  final String resultCode;

  @override
  String toString() => 'PayWithMpurseResult: $resultCode';
}
